'use strict';

exports.__esModule = true;
exports.default = Composer;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function Composer({
  components = [],
  children,
  renderPropName,
  mapResult
}) {
  if (typeof children !== 'function') {
    return null;
  }

  // This is the argument that we pass into `children`.
  const results = [];

  function chainComponents(childrenComponents) {
    // When we reach the end of our `childrenComponents`, we can render out
    // the response array.
    if (childrenComponents.length === 0) {
      return children([...results]);
    }

    const index = components.length - childrenComponents.length;

    const component =
    // Each props.components entry is either an element or function [element factory]
    // When it is a function, produce an element by invoking it with currently accumulated results.
    typeof components[index] === 'function' ? components[index]([...results]) : components[index];

    // We create a clone of the childrenComponents so that subsequent calls to `chidlren`
    // render the same tree. If we modified `reversedComponents` directly, then the tree would
    // be different with each call to `children`.
    const childrenComponentsClone = [...childrenComponents];
    childrenComponentsClone.pop();

    return _react2.default.cloneElement(component, {
      [renderPropName]() {
        if (mapResult) {
          results[index] = mapResult.apply(null, arguments);
        } else {
          results[index] = arguments[0];
        }
        return chainComponents(childrenComponentsClone);
      }
    });
  }

  return chainComponents(components);
}

Composer.propTypes = {
  children: _propTypes2.default.func,
  components: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.element, _propTypes2.default.func])),
  renderPropName: _propTypes2.default.string,
  mapResult: _propTypes2.default.func
};

Composer.defaultProps = {
  renderPropName: 'children'
};